/* Copyright (c) 2005 PrimeBase Technologies GmbH
 *
 * PrimeBase XT
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * 2005-05-24	Paul McCullagh
 *
 * H&G2JCtL
 */
#ifndef __xt_cache_h__
#define __xt_cache_h__

//#define XT_USE_MYSYS

#include "filesys_xt.h"
#include "index_xt.h"

struct XTOpenTable;
struct XTIdxReadBuffer;

#ifdef DEBUG
//#define XT_USE_CACHE_DEBUG_SIZES
#endif

#ifdef XT_USE_CACHE_DEBUG_SIZES
#define XT_INDEX_CACHE_SEGMENT_SHIFTS		1
#else
#define XT_INDEX_CACHE_SEGMENT_SHIFTS		3
#endif

#define IDX_CAC_BLOCK_FREE			0
#define IDX_CAC_BLOCK_CLEAN			1
#define IDX_CAC_BLOCK_DIRTY			2

typedef struct XTIndBlock {
	xtIndexNodeID		cb_address;						/* The block address. */
	u_int				cb_file_id;						/* The file id of the block. */
	xtWord2				cb_state;						/* Block status. */
	xtWord2				cp_flush_seq;
	xtWord4				cb_ru_time;						/* If this is in the top 1/4 don't change position in MRU list. */
	struct XTIndBlock	*cb_next;						/* Pointer to next block on hash list, or next free block on free list. */
	struct XTIndBlock	*cb_dirty_next;					/* Double link for dirty blocks, next pointer. */
	struct XTIndBlock	*cb_dirty_prev;					/* Double link for dirty blocks, previous pointer. */
	struct XTIndBlock	*cb_mr_used;					/* More recently used blocks. */
	struct XTIndBlock	*cb_lr_used;					/* Less recently used blocks. */
#ifdef XT_USE_DIRECT_IO_ON_INDEX
	xtWord1				*cb_data;
#else
	xtWord1				cb_data[XT_INDEX_PAGE_SIZE];
#endif
} XTIndBlockRec, *XTIndBlockPtr;

typedef struct XTIndFreeBlock {
	XTDiskValue1			if_status_1;
	XTDiskValue1			if_unused1_1;
	XTDiskValue2			if_unused2_2;
	XTDiskValue4			if_unused3_4;
	XTDiskValue8			if_next_block_8;
} XTIndFreeBlockRec, *XTIndFreeBlockPtr;

void	xt_ind_init(XTThreadPtr self, size_t cache_size);
void	xt_ind_exit(XTThreadPtr self);
xtInt8	xt_ind_get_usage();
xtInt8	xt_ind_get_size();
xtBool	xt_ind_write(struct XTOpenTable *ot, XTIndexPtr ind, xtIndexNodeID offset, size_t size, xtWord1 *data);
xtBool	xt_ind_write_to_offset(struct XTOpenTable *ot, XTIndexPtr ind, xtIndexNodeID address, size_t offset, size_t size, xtWord1 *data);
xtBool	xt_ind_write_cache(struct XTOpenTable *ot, xtIndexNodeID offset, size_t size, xtWord1 *data);
xtBool	xt_ind_clean(struct XTOpenTable *ot, XTIndexPtr ind, xtIndexNodeID offset);
xtBool	xt_ind_read_bytes(struct XTOpenTable *ot, xtIndexNodeID offset, size_t size, xtWord1 *data);
xtBool	xt_ind_read(struct XTOpenTable *ot, xtIndexNodeID offset, struct XTIdxBranch *rb);
xtBool	xt_ind_search(struct XTOpenTable *ot, xtIndexNodeID address, XTScanBranchFunc func, XTIndexPtr ind, XTIdxKeyValuePtr value, XTIdxResultRec *result);
void	xt_ind_check_cache(XTIndexPtr ind);
xtBool	xt_ind_reserve(struct XTOpenTable *ot, u_int count);
void	xt_ind_free_reserved(struct XTOpenTable *ot);
void	xt_ind_unreserve(struct XTOpenTable *ot);

#ifdef DEBUG
//#define DEBUG_CHECK_IND_CACHE
#endif

//#define XT_TRACE_INDEX

#ifdef XT_TRACE_INDEX
#define IDX_TRACE(x, y, z)		xt_trace(x, y, z)
#else
#define IDX_TRACE(x, y, z)
#endif

#endif
